import { createComponent as _$createComponent, memo as _$memo } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { useTsp } from "@typespec/emitter-framework";
import { FunctionDeclaration } from "@typespec/emitter-framework/typescript";
import { reportDiagnostic } from "../../lib.js";
import { buildClientParameters } from "../../utils/parameters.js";
import { httpRuntimeTemplateLib } from "../external-packages/ts-http-runtime.js";
import { addClientTestOptions } from "../testing/client-options.js";
import { getClientcontextDeclarationRef } from "./client-context-declaration.js";
import { ParametrizedEndpoint } from "./parametrized-endpoint.js";
export function getClientContextFactoryRef(client) {
  return ay.refkey(client, "contextFactory");
}
export function ClientContextFactoryDeclaration(props) {
  const {
    $
  } = useTsp();
  const ref = getClientContextFactoryRef(props.client);
  const contextDeclarationRef = getClientcontextDeclarationRef(props.client);
  const namePolicy = ts.useTSNamePolicy();
  const factoryFunctionName = namePolicy.getName(`create_${props.client.name}Context`, "function");
  const clientConstructor = $.client.getConstructor(props.client);
  const parameters = buildClientParameters(props.client);
  const urlTemplate = $.client.getUrlTemplate(props.client);
  const endpointRef = ay.refkey();
  const resolvedEndpoint = _$createComponent(ParametrizedEndpoint, {
    refkey: endpointRef,
    get template() {
      return urlTemplate.url;
    },
    get params() {
      return urlTemplate.parameters;
    }
  });
  let credentialsRef;
  if (parameters.some(p => p.name === "credential")) {
    credentialsRef = "credential";
  }

  // Filter out optional parameters, they will be passed as options
  const args = _$createComponent(ClientFactoryArguments, {
    get client() {
      return props.client;
    },
    endpointRef: endpointRef,
    credentialsRef: credentialsRef
  });
  return _$createComponent(FunctionDeclaration, {
    "export": true,
    name: factoryFunctionName,
    type: clientConstructor,
    returnType: contextDeclarationRef,
    refkey: ref,
    parametersMode: "replace",
    parameters: parameters,
    get children() {
      return [resolvedEndpoint, "return ", _$createComponent(ts.FunctionCallExpression, {
        get target() {
          return httpRuntimeTemplateLib.getClient;
        },
        args: [args]
      })];
    }
  });
}
function ClientFactoryArguments(props) {
  const params = [[_$memo(() => props.endpointRef), ","]];
  return [...params, props.credentialsRef ? _$createComponent(ClientOptionsExpression, {
    get children() {
      return [_$memo(() => props.credentialsRef), _$createComponent(AuthSchemeOptions, {
        get client() {
          return props.client;
        }
      })];
    }
  }) : _$createComponent(ClientOptionsExpression, {})];
}
function AuthScheme(props) {
  const {
    $
  } = useTsp();
  switch (props.scheme.type) {
    case "http":
      return _$createComponent(ts.ObjectExpression, {
        get children() {
          return _$createComponent(ay.List, {
            comma: true,
            get children() {
              return [_$createComponent(ts.ObjectProperty, {
                name: "kind",
                jsValue: "http"
              }), _$createComponent(ts.ObjectProperty, {
                name: "scheme",
                get jsValue() {
                  return props.scheme.scheme.toLowerCase();
                }
              })];
            }
          });
        }
      });
    case "apiKey":
      if (props.scheme.in !== "header") {
        reportDiagnostic($.program, {
          code: "non-model-parts",
          target: props.client.service
        });
        return null;
      }
      return _$createComponent(ts.ObjectExpression, {
        get children() {
          return _$createComponent(ay.List, {
            comma: true,
            get children() {
              return [_$createComponent(ts.ObjectProperty, {
                name: "kind",
                jsValue: "apiKey"
              }), _$createComponent(ts.ObjectProperty, {
                name: "apiKeyLocation",
                get jsValue() {
                  return props.scheme.in;
                }
              }), _$createComponent(ts.ObjectProperty, {
                name: "name",
                get jsValue() {
                  return props.scheme.name;
                }
              })];
            }
          });
        }
      });
    case "oauth2":
      return _$createComponent(ts.ObjectExpression, {
        get children() {
          return _$createComponent(ay.List, {
            comma: true,
            get children() {
              return [_$createComponent(ts.ObjectProperty, {
                name: "kind",
                jsValue: "oauth2"
              }), _$createComponent(ts.ObjectProperty, {
                name: "flows",
                get children() {
                  return ["[", _$createComponent(ay.For, {
                    get each() {
                      return props.scheme.flows;
                    },
                    comma: true,
                    line: true,
                    children: flow => _$createComponent(OAuth2Flow, {
                      flow: flow
                    })
                  }), "]"];
                }
              })];
            }
          });
        }
      });
    default:
      return null;
  }
}
function OAuth2Flow(props) {
  // rename type to kind and clean up the scopes
  const {
    type,
    ...rewrittenFlow
  } = {
    kind: props.flow.type,
    ...props.flow,
    // scopes are sometimes duplicated so converting to set and back to dedupe
    scopes: [...new Set(props.flow.scopes.map(s => s.value))]
  };
  return _$createComponent(ts.ObjectExpression, {
    jsValue: rewrittenFlow
  });
}
function AuthSchemeOptions(props) {
  const {
    $
  } = useTsp();
  const clientCredential = $.client.getAuth(props.client);
  if (clientCredential.schemes.length === 0) {
    return null;
  }

  // Filtering out custom http schemes for "http/custom" spector scenario
  // TODO: Should typekit allow for arbitrary strings on scheme with http auth type?
  const supportedSchemes = clientCredential.schemes.filter(s => s.type !== "http" || ["Basic", "Bearer"].includes(s.scheme));
  return _$createComponent(ts.ObjectProperty, {
    name: "authSchemes",
    get children() {
      return ["[", _$createComponent(ay.For, {
        each: supportedSchemes,
        comma: true,
        line: true,
        children: scheme => _$createComponent(AuthScheme, {
          scheme: scheme,
          get client() {
            return props.client;
          }
        })
      }), "]"];
    }
  });
}
function ClientOptionsExpression(props) {
  const options = ["...options"];

  // Conditionally add test options
  // based on the environment variable TYPESPEC_JS_EMITTER_TESTING
  addClientTestOptions(options);
  const children = Array.isArray(props.children) ? props.children : [props.children];
  if (children.length) {
    const children = Array.isArray(props.children) ? props.children : [props.children];
    options.push(...children);
  }
  return _$createComponent(ts.ObjectExpression, {
    get children() {
      return _$createComponent(ay.For, {
        each: options,
        joiner: ",",
        line: true,
        children: child => child
      });
    }
  });
}