import { memo as _$memo, createComponent as _$createComponent } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { useTransformNamePolicy } from "@typespec/emitter-framework";
import { getDefaultValue, hasDefaultValue } from "../utils/parameters.js";
import { JsonTransform } from "./transforms/json/json-transform.js";
export function HttpRequestParametersExpression(props) {
  const parameters = [];
  const transformNamer = useTransformNamePolicy();
  if (props.children || Array.isArray(props.children) && props.children.length) {
    parameters.push([_$memo(() => props.children), ","]);
  }
  if (!props.parameters && parameters.length) {
    return _$createComponent(ts.ObjectExpression, {
      children: parameters
    });
  } else if (!props.parameters) {
    return _$createComponent(ts.ObjectExpression, {});
  }
  const optionsParamRef = props.optionsParameter ?? "options";
  const members = _$createComponent(ay.For, {
    get each() {
      return props.parameters;
    },
    line: true,
    comma: true,
    children: httpProperty => {
      const defaultValue = getDefaultValue(httpProperty);
      const propertyExpression = buildMemberChainExpression(props.httpOperation, httpProperty, optionsParamRef);
      const transportParamName = transformNamer.getTransportName(httpProperty.property);
      if (defaultValue) {
        const paramValue = [_$memo(() => propertyExpression.fullExpression), ` ?? ${defaultValue}`];
        return _$createComponent(ay.NamePolicyContext.Provider, {
          value: {
            getName: n => n
          },
          get children() {
            return _$createComponent(ts.ObjectProperty, {
              name: transportParamName,
              value: paramValue
            });
          }
        });
      }
      if (propertyExpression.isNullish) {
        return ay.code`
        ...(${propertyExpression.fullExpression} && {${_$createComponent(JsonTransform, {
          get itemRef() {
            return propertyExpression.leadingExpression;
          },
          get type() {
            return httpProperty.property;
          },
          target: "transport"
        })}})
      `;
      } else {
        return _$createComponent(JsonTransform, {
          get itemRef() {
            return propertyExpression.leadingExpression;
          },
          get type() {
            return httpProperty.property;
          },
          target: "transport"
        });
      }
    }
  });
  parameters.push(members);
  return _$createComponent(ts.ObjectExpression, {
    children: parameters
  });
}
/**
 * Builds the member chain expression for potentially nested HTTP parameters.
 * This logic will be simplified and made more robust once a couple of changes land in Alloy.
 *    - Support refkeys on interface members
 *    - Support for conditional chaining on ts.MemberChainExpression
 *    - Eventually we'll have an API like:
 * @example
 * ```tsx
 *        <InstancePropertyExpression
 *            instance={refkey(parameter)}
 *            staticMember={refkey(property)}
 *        />
 * ```
 * @param httpOperation - The HttpOperation containing parameters.
 * @param httpProperty - The current HttpProperty to build an access path for.
 * @param optionsParamRef - Reference to the user's main options parameter.
 * @returns The full nested expression, leading expression, nullish flag, and final property name.
 */
function buildMemberChainExpression(httpOperation, httpProperty, optionsParamRef) {
  const segments = [];
  let isNullish = httpProperty.property.optional === true;
  const namePolicy = ay.useNamePolicy();
  const propertyApplicationName = namePolicy.getName(httpProperty.property.name, "property");

  // If the property is at the top level, return early
  if (httpProperty.path.length === 1) {
    if (httpProperty.property.optional || hasDefaultValue(httpProperty)) {
      return {
        propertyName: propertyApplicationName,
        isNullish,
        fullExpression: ay.code`${optionsParamRef}?.${propertyApplicationName}`,
        leadingExpression: ay.code`${optionsParamRef}`
      };
    }
    return {
      propertyName: propertyApplicationName,
      isNullish,
      fullExpression: propertyApplicationName
    };
  }

  // Build access segments for nested properties.
  let currentPath = httpProperty.path.slice(0, -1);
  while (currentPath.length > 0) {
    const parentProperty = findHttpPropertyByPath(httpOperation, currentPath);
    if (!parentProperty) {
      break;
    }

    // Use optional chaining if the parent is optional.
    const joiner = parentProperty.property.optional ? "?." : ".";
    if (parentProperty.property.optional) {
      isNullish = true;
    }
    const parentName = namePolicy.getName(parentProperty.property.name, "property");

    // Unshift the joiner first
    segments.unshift(joiner);

    // Use direct property access if possible.
    if (isValidIdentifier(parentName)) {
      segments.unshift(ay.code`${parentName}`);
    } else {
      // For non-valid identifiers, use bracket notation.
      segments.unshift(`[${JSON.stringify(parentName)}]`);
    }

    // Remove the last element to move one level up.
    currentPath = currentPath.slice(0, -1);
  }
  const fullExpression = [_$createComponent(ay.List, {
    children: segments
  }), propertyApplicationName];
  const leadingExpression = _$createComponent(ay.List, {
    get children() {
      return segments.slice(0, -1);
    }
  });
  return {
    propertyName: propertyApplicationName,
    isNullish,
    fullExpression,
    leadingExpression
  };
}
function isValidIdentifier(name) {
  return /^[A-Za-z_$][A-Za-z0-9_$]*$/.test(name);
}
function findHttpPropertyByPath(httpOperation, path) {
  for (const property of httpOperation.parameters.properties) {
    // Check if the property path matches the given path
    if (JSON.stringify(property.path) === JSON.stringify(path)) {
      return property;
    }
  }
  return undefined;
}