import { createComponent as _$createComponent, memo as _$memo } from "@alloy-js/core/jsx-runtime";
import * as ay from "@alloy-js/core";
import * as ts from "@alloy-js/typescript";
import { useTsp } from "@typespec/emitter-framework";
import { FunctionDeclaration } from "@typespec/emitter-framework/typescript";
import * as cl from "@typespec/http-client";
import { reportDiagnostic } from "../../../lib.js";
import { getClientcontextDeclarationRef } from "../../client-context/client-context-declaration.js";
import { OperationOptionsDeclaration } from "../../operation-options.js";
import { getOperationParameters } from "../../operation-parameters.js";
import { getBuildPagedAsyncIteratorRefkey, getPagedAsyncIterableIteratorRefkey } from "../../static-helpers/paging-helper.js";
import { getPageItemTypeName } from "../paging/page-item.js";
import { getPageResponseTypeRefkey, PageResponseDeclaration } from "../paging/page-response.js";
import { getPageSettingProperties, getPageSettingsTypeRefkey, PageSettingsDeclaration } from "../paging/page-settings.js";
import { HttpRequestSend } from "../paging/request-send.js";
import { getHttpRequestDeserializeRefkey, HttpResponseDeserialize } from "../paging/response-deserialize.js";
import { extractPagingDetail } from "../paging/util.js";
export const PaginatedOperationHandler = {
  canHandle(httpOperation) {
    const {
      $
    } = useTsp();
    const pagingMetadata = $.operation.getPagingMetadata(httpOperation.operation);
    return pagingMetadata !== undefined;
  },
  handle(httpOperation) {
    const {
      $
    } = useTsp();
    const clientLibrary = cl.useClientLibrary();
    const client = clientLibrary.getClientForOperation(httpOperation);
    if (!client) {
      reportDiagnostic($.program, {
        code: "client-not-found",
        target: httpOperation.operation
      });
      return;
    }
    const pagingOperation = $.operation.getPagingMetadata(httpOperation.operation);
    const responseRefkey = ay.refkey(httpOperation, "http-response");
    const operationRefkey = ay.refkey(httpOperation.operation);
    const returnType = ay.code`${getPagedAsyncIterableIteratorRefkey()}<${getPageItemTypeName(pagingOperation)},${getPageResponseTypeRefkey(httpOperation)},${getPageSettingsTypeRefkey(httpOperation)}>`;
    const clientContextInterfaceRef = getClientcontextDeclarationRef(client);
    const signatureParams = [{
      name: "client",
      type: clientContextInterfaceRef,
      refkey: ay.refkey(client, "client")
    }, ...getOperationParameters(httpOperation)];
    const pagingDetail = extractPagingDetail(httpOperation, pagingOperation);
    // Exclude from operation options and include them into PageSettings
    const excludes = getPageSettingProperties(pagingOperation).map(p => p.property);
    return _$createComponent(ay.List, {
      get children() {
        return [_$createComponent(OperationOptionsDeclaration, {
          operation: httpOperation,
          excludes: excludes
        }), _$createComponent(PageSettingsDeclaration, {
          operation: httpOperation,
          pagingOperation: pagingOperation
        }), _$createComponent(PageResponseDeclaration, {
          operation: httpOperation,
          pagingOperation: pagingOperation
        }), _$createComponent(HttpRequestSend, {
          httpOperation: httpOperation,
          responseRefkey: responseRefkey
        }), _$createComponent(HttpResponseDeserialize, {
          httpOperation: httpOperation,
          responseRefkey: responseRefkey
        }), _$createComponent(FunctionDeclaration, {
          "export": true,
          get name() {
            return httpOperation.operation.name;
          },
          returnType: returnType,
          refkey: operationRefkey,
          parametersMode: "replace",
          parameters: signatureParams,
          get children() {
            return _$createComponent(ay.List, {
              get children() {
                return [_$createComponent(GetElements, {
                  httpOperation: httpOperation,
                  pageDetail: pagingDetail
                }), _$createComponent(GetPagedResponse, {
                  httpOperation: httpOperation,
                  pageDetail: pagingDetail,
                  operationParams: signatureParams
                }), [_$memo(() => ay.code`
              return ${getBuildPagedAsyncIteratorRefkey}<${getPageItemTypeName(pagingOperation)}, ${getPageResponseTypeRefkey(httpOperation)}, ${getPageSettingsTypeRefkey(httpOperation)}>({${getElementsRefkey(httpOperation)}, ${getPagedResponseFunctionRefkey(httpOperation)}});`)]];
              }
            });
          }
        })];
      }
    });
  }
};
function getPagedResponseFunctionRefkey(httpOperation) {
  return ay.refkey(httpOperation, "get-paged-response");
}
function GetPagedResponse(props) {
  const pagingDetail = props.pageDetail;
  const httpOperation = props.httpOperation;
  const params = [{
    name: "nextToken",
    type: "string",
    optional: true,
    refkey: ay.refkey()
  }, {
    name: "settings",
    type: getPageSettingsTypeRefkey(httpOperation),
    optional: true,
    refkey: ay.refkey()
  }];
  // TODO: template can't resolve the ref key so fallback to useTSNamePolicy
  const namePolicy = ts.useTSNamePolicy();
  const functionName = namePolicy.getName(httpOperation.operation.name + "Send", "function");
  const sendParamStr = props.operationParams.map(param => param.name).slice(0, -1).join(", ");
  return _$createComponent(FunctionDeclaration, {
    name: "getPagedResponse",
    async: true,
    get refkey() {
      return getPagedResponseFunctionRefkey(httpOperation);
    },
    parameters: params,
    get children() {
      return ay.code` 
      ${props.pageDetail.pattern === "nextLink" ? `
        let response: PathUncheckedResponse;
        if (nextToken) {
          response = await client.pathUnchecked(nextToken).get();
        } else {
          const combinedOptions = { ...options, ...settings };
          response = await ${functionName}(${sendParamStr}, combinedOptions);
        } ` : `
        const combinedOptions = {...options, ...settings};
        ${pagingDetail.input?.nextToken ? `
            if (nextToken) {
              combinedOptions.${pagingDetail.input?.nextToken} = nextToken;
            }` : ``}
        const response = await ${functionName}(${sendParamStr},  combinedOptions);    
        `}
    return {
      pagedResponse: await ${getHttpRequestDeserializeRefkey(httpOperation)}(response, options),
      nextToken: ${pagingDetail.output.nextToken ? `response.${pagingDetail.output.nextToken.position}["${pagingDetail.output.nextToken.name}"]` : `undefined`},
    };
      `;
    }
  });
}
function getElementsRefkey(httpOperation) {
  return ay.refkey(httpOperation, "get-elements");
}
function GetElements(props) {
  const pagingDetail = props.pageDetail;
  const httpOperation = props.httpOperation;
  const params = [{
    name: "response",
    type: getPageResponseTypeRefkey(httpOperation),
    refkey: ay.refkey()
  }];
  return _$createComponent(FunctionDeclaration, {
    name: "getElements",
    get refkey() {
      return getElementsRefkey(httpOperation);
    },
    parameters: params,
    get children() {
      return ay.code`
      return response.${pagingDetail.output.items};
      `;
    }
  });
}